from movva_tools.models.organization_models import RapidProOrganization
from movva_tools.exceptions import ObjectDoesNotExistException
from movva_tools.services.base_service import BaseService


class OrganizationService(BaseService):

    def __init__(
        self,
        _user=None, _host=None, _port=None, _db_name=None, _password=None,
        db_connection=None
    ) -> None:

        super().__init__(
            _user=_user, _password=_password,
            _host=_host, _port=_port,
            _db_name=_db_name,
            db_connection=db_connection
        )

        self.org = RapidProOrganization

    def get_org_by_name(self, org_name):

        org = self.db_connection.session.query(self.org).filter_by(
            name=org_name
        ).first()

        if org:
            return org
        else:
            raise ObjectDoesNotExistException(dababase_object=self.org)

    def get_org_by_id(self, org_id):

        org = self.db_connection.session.query(self.org).filter_by(
            id=org_id
        ).first()

        if org:
            return org
        else:
            raise ObjectDoesNotExistException(dababase_object=self.org)

    def get_token_org(self, db_connection=None):
        pass



from movva_tools.models.campaign_models import RapidProCampaigns
from movva_tools.serializers.base_serializer import (
    BaseSerializer, serializer_fields, date_field_format
)


class CampaignSerializer(BaseSerializer):

    class Meta:
        model = RapidProCampaigns

    uuid = serializer_fields.UUID(dump_only=True)
    name = serializer_fields.String(dump_only=True)
    created_on = date_field_format
    is_active = serializer_fields.Boolean(dump_only=True)



from movva_tools.services.base_service import BaseService
from movva_tools.models.contacts_models import (
    RapidProContactFields, RapidProContacts,
    RapidProContactGroups, RapidProContactGroupsContacts
)
from movva_tools.exceptions import ObjectDoesNotExistException
from movva_tools.models.contacts_models import RapidProContactURN
from sqlalchemy import or_


class ContactService(BaseService):

    def __init__(
        self,
        _user=None, _host=None, _port=None, _db_name=None, _password=None,
        db_connection=None
    ) -> None:

        super().__init__(
            _user=_user, _password=_password,
            _host=_host, _port=_port,
            _db_name=_db_name,
            db_connection=db_connection
        )

        # model table entities
        self.Contact = RapidProContacts
        self.ContactFields = RapidProContactFields
        self.ContactGroups = RapidProContactGroups
        self.ContactGroupsContacts = RapidProContactGroupsContacts

    def get_contact_group_by_id(self, id):
        contact_group = self.db_connection.session.query(
            self.ContactGroups
        ).filter_by(
            id=id
        ).first()

        if contact_group:
            return contact_group
        else:
            raise ObjectDoesNotExistException(
                dababase_object=self.ContactGroups
            )

    def get_contact_group_by_name(self, name, org_id):
        contact_group = self.db_connection.session.query(
            self.ContactGroups
        ).filter_by(
            name=name,
            org_id=org_id
        ).first()

        if contact_group:
            return contact_group
        else:
            raise ObjectDoesNotExistException(
                dababase_object=self.ContactGroups
            )

    def get_contact_groups_contact_by_contact_id(self, contact_id_list: list):
        contacts_groups_contacts = self.db_connection.session.query(
            self.ContactGroupsContacts
        ).filter(
            self.ContactGroupsContacts.contact_id.in_(contact_id_list)
        )

        groups = []
        contact_ids = []
        for contact_group_contact in contacts_groups_contacts:
            groups.append(
                contact_group_contact.contactgroup_id
            )
            contact_ids.append(
                contact_group_contact.contact_id
            )

        return contact_ids, groups

    def get_contacfield_by_label(self, org, label):
        contact_field = self.db_connection.session.query(
            self.ContactFields
        ).filter_by(
            label=label,
            org_id=org.id
        ).first()

        if contact_field:
            return contact_field
        else:
            raise ObjectDoesNotExistException(
                dababase_object=self.ContactFields
            )


class ContactURNService(BaseService):

    def __init__(
        self,
        _user=None, _host=None, _port=None, _db_name=None, _password=None,
        db_connection=None
    ) -> None:

        super().__init__(
            _user=_user, _password=_password,
            _host=_host, _port=_port,
            _db_name=_db_name,
            db_connection=db_connection
        )

        self.ContactURN = RapidProContactURN

    def get_contact_urn_by_contact_id(self, contact_id):

        contact_urn = self.db_connection.session.query(
            self.ContactURN
        ).filter_by(
            contact_id=contact_id
        ).first()

        if not contact_urn:
            raise ObjectDoesNotExistException(
                dababase_object=self.ContactURN
            )

        return contact_urn

    def batch_get_contact_urn_by_contact_id(self, contact_id_list: list):

        contact_urns = self.db_connection.session.query(
            self.ContactURN
        ).filter(
            self.ContactURN.contact_id.in_(contact_id_list)
        )

        if not contact_urns:
            raise ObjectDoesNotExistException(
                dababase_object=self.ContactURN
            )

        return contact_urns

    def batch_get_contact_urn_by_identity(self, identities_list: list):

        contact_urns = self.db_connection.session.query(
            self.ContactURN
        ).filter(
            or_(
                *[self.ContactURN.identity.like(f'%{identity}%') for identity in identities_list]  # noqa
            )
        )

        if not contact_urns:
            raise ObjectDoesNotExistException(
                dababase_object=self.ContactURN
            )

        return contact_urns



from google.oauth2.credentials import Credentials
from googleapiclient.discovery import build
from googleapiclient.http import MediaIoBaseDownload
import io
import pandas as pd
import logging


logging.basicConfig(
    level=logging.INFO,
    format=f"%(levelname)-8s %(name)s: \n %(message)s"
)
logger = logging.getLogger(name='GoogleDriveLogger')
logger.setLevel(level=logging.INFO)


class GoogleDriveIntegration:
    def __init__(self, folder_id, mimeType, credentials) -> None:
        self.folder_id = folder_id
        self.mime_type = mimeType
        self.credentials = self._set_credentials(credentials)
        self.drive_service = self._set_drive_service()

        self._df = None
        self._logger = logger

    def _set_credentials(self, credentials):
        # Configurar as credenciais de autenticação
        return Credentials.from_authorized_user_info(credentials)

    def _set_drive_service(self):
        return build('drive', 'v3', credentials=self.credentials)

    def fetch_files(self):
        try:
            results = self.drive_service.files().list(
                q=f"'{self.folder_id}' in parents and mimeType='{self.mime_type}'",
                fields='files(id, name)'
            ).execute()

            files = results.get('files', [])
            return files
        except Exception as e:
            self._logger.error(f"Erro ao buscar arquivos do Google Drive: {e}")
            return []

    def download_file(self, file):
        try:
            file_id = file['id']

            request = self.drive_service.files().get_media(fileId=file_id)
            fh = io.BytesIO()
            downloader = MediaIoBaseDownload(fh, request)
            done = False
            while not done:
                status, done = downloader.next_chunk()

            self._df = pd.read_csv(io.StringIO(fh.getvalue().decode('utf-8')))
        except Exception as e:
            self._logger.error(f"Erro ao baixar o arquivo do Google Drive: {e}")

    def rename_file(self, file):
        try:
            file_name = file['name']
            file_id = file['id']

            new_name = f"PROCESSADO_{file_name}"
            file_metadata = {'name': new_name}
            self.drive_service.files().update(
                fileId=file_id, body=file_metadata
            ).execute()
        except Exception as e:
            self._logger.error(f"Erro ao renomear o arquivo no Google Drive: {e}")



from movva_tools.databases import RapidProDatabase
from sqlalchemy.exc import SQLAlchemyError


class BaseService:

    def __init__(
        self,
        _user=None, _host=None, _port=None, _db_name=None, _password=None,
        db_connection=None
    ) -> None:
        if not db_connection:
            self.db_connection = self.__create_default_connection(
                _user=_user,
                _password=_password,
                _host=_host,
                _port=_port,
                _db_name=_db_name
            )
        else:
            self.db_connection = db_connection

    def __create_default_connection(self, _user, _host, _port, _db_name, _password):  # noqa
        db = RapidProDatabase(
            _user=_user,
            _password=_password,
            _host=_host,
            _port=_port,
            _db_name=_db_name
        )
        db.set_session()
        return db

    def add(self, object):
        self.db_connection.session.add(object)

    def commit(self):
        self.db_connection.session.commit()

    def flush(self):
        """
            Flush para obter o ID gerado sem commit
        """
        self.db_connection.session.flush()

    def rollback_session(self):
        try:
            self.db_connection.session.rollback()
            return True
        except SQLAlchemyError as e:
            raise Exception(e)

    def close_session(self):
        self.db_connection.session.close()



from movva_tools.exceptions import ObjectDoesNotExistException
from movva_tools.models.messages_models import RapidProMessages
from movva_tools.models.contacts_models import RapidProContacts, RapidProContactURN
from movva_tools.services.base_service import BaseService


class MessageService(BaseService):
    def __init__(
        self,
        _user=None, _host=None, _port=None, _db_name=None, _password=None,
        db_connection=None
    ) -> None:

        super().__init__(
            _user=_user, _password=_password,
            _host=_host, _port=_port,
            _db_name=_db_name,
            db_connection=db_connection
        )
        self.event_loop = None

        # model table entities
        self.Message = RapidProMessages
        self.Contact = RapidProContacts
        self.ContactURN = RapidProContactURN

    def fetch_message_by_id(self, message_id):
        message = self.db_connection.session.query(self.Message).filter_by(
            id=message_id
        ).first()

        if message:
            return message
        else:
            raise ObjectDoesNotExistException(dababase_object=self.Message)

    def fetch_messages_by_channel_and_org(self, channel_id, org_id, direction, status=None):
        if status:
            messages = self.db_connection.session.query(self.Message).filter_by(
                status=status,
                direction=direction,
                channel_id=channel_id,
                org_id=org_id
            ).all()
        else:
            messages = self.db_connection.session.query(self.Message).filter_by(
                channel_id=channel_id,
                direction=direction,
                org_id=org_id
            ).all()

        return messages if messages else []

    async def create(
        self, session, text, direction, status, visibility, msg_type, created_on, modified_on,
        sent_on, queued_on, channel_id, contact_id, contact_urn_id, org_id, topup_id,
        failed_reason, flow_id, next_attempt, external_id, attachments=None, msg_count=1,
        error_count=0, high_priority=False, contact_ids=[]
    ):
        async with session as async_session:
            message = self.Message(
                text=text,
                direction=direction,
                status=status,
                visibility=visibility,
                msg_type=msg_type,
                created_on=created_on,
                modified_on=modified_on,
                sent_on=sent_on,
                queued_on=queued_on,
                channel_id=channel_id,
                contact_id=contact_id,
                contact_urn_id=contact_urn_id,
                org_id=org_id,
                topup_id=topup_id,
                failed_reason=failed_reason,
                flow_id=flow_id,
                next_attempt=next_attempt,
                external_id=external_id,
                attachments=attachments,
                msg_count=msg_count,
                error_count=error_count,
                high_priority=high_priority,
                contact_ids=contact_ids
            )
            await async_session.add(message)
        await async_session.commit()




from movva_tools.constants import DEFAULT_USER_ID
from movva_tools.models.user_models import RapidProUser
from movva_tools.exceptions import ObjectDoesNotExistException
from movva_tools.services.base_service import BaseService


class UserService(BaseService):

    def __init__(
        self,
        _user=None, _host=None, _port=None, _db_name=None, _password=None,
        db_connection=None
    ) -> None:

        super().__init__(
            _user=_user, _password=_password,
            _host=_host, _port=_port,
            _db_name=_db_name,
            db_connection=db_connection
        )

        # model table entities
        self.user = RapidProUser

    def get_default_user(self):

        default_user = self.db_connection.session.query(self.user).filter_by(
            id=DEFAULT_USER_ID
        ).first()

        if default_user:
            return default_user
        else:
            raise ObjectDoesNotExistException(dababase_object=self.user)
